<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Error;
use Util\Validation;
use SCM\Traits\TResponse;

class Inventarios extends \Zeedhi\Framework\Controller\Simple {
    
    use TResponse;

	protected $inventarios;
	protected $environment;
	
	public function __construct(\SCM\Services\Inventarios $inventarios, $environment) {
        $this->inventarios = $inventarios;
        $this->environment = $environment;
    }

	public function retornaUltimoInventario(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];

            $ultimoInventario = $this->inventarios->retornaUltimoInventario($nrorg, $cdfilial);

			$response->addDataSet(new DataSet('UltimoInventario', $ultimoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoInventario(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];

            $proximoCodigoInventario = $this->inventarios->retornaProximoCodigoInventario($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('ProximoCodigoInventario', $proximoCodigoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoProdutoInventario(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];

            $proximoCodigoProdutoInventario = $this->inventarios->retornaProximoCodigoProdutoInventario($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('ProximoCodigoProdutoInventario', $proximoCodigoProdutoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaDatasInventarios(Request $request, Response $response) {
        try {
            // $filter = $request->getFilterCriteria()->getConditions();
            $params = $request->getParameters();
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $params[0]['value'];

            $datasInventario = $this->inventarios->retornaDatasInventarios($nrorg, $cdfilial);
            
            $this->sendResponse($datasInventario);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosInventarios(Request $request, Response $response) {
        try {
            // $filter = $request->getFilterCriteria()->getConditions();
            $params = $request->getParameters();
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $params[0]['value'];
            $nrinvent = $params[1]['value'];

            $produtosInventario = $this->inventarios->retornaProdutosInventarios($nrorg, $cdfilial, $nrinvent);

            // $response->addDataSet(new DataSet('ProdutosInventario', $produtosInventario));
            $this->sendResponse($produtosInventario);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criarInventario(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            
            $inventariosSalvos = $this->inventarios->criarInventario($rows);
            $nrorg = $this->environment->getNrorg();
            if(sizeof($inventariosSalvos) > 0) {
                    
                $this->inventarios->adicionarProdutos($inventariosSalvos, $nrorg);
            }

        } catch (\Exception $e) {
            
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function excluirInventario(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrorg();
            $inventariosSalvos = $this->inventarios->excluirInventario($rows[0], $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarProduto(Request\Dataset $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrorg();
            $this->inventarios->salvarProduto($rows[0], $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function excluirProduto(Request\Dataset $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrorg();
            $this->inventarios->excluirProduto($rows[0], $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}