<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class Solicitacao extends \Zeedhi\Framework\Controller\Simple {

	protected $solicitacao;
	protected $environment;

	public function __construct(\SCM\Services\Solicitacao $solicitacao, $environment) {
        $this->solicitacao = $solicitacao;
        $this->environment = $environment;
    }

	public function retornaDatasEntregaSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $datasEntraga = $this->solicitacao->retornaDatasEntregaSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('DatasEntregaSolicitacao', $datasEntraga));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosSolicitacao(Request $request, Response $response) {
        try {
        // 	$filter = $request->getFilterCriteria()->getConditions();
            $row = $request->getRow();
            
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $filiais = $row['filiais'];
            $dataInicial = $row['dataInicial'];
            $dataFinal = $row['dataFinal'];
            $periodo = $row['periodo'];
            
            $produtos = [];
            foreach($filiais as $filial) {
                $produtos[] = $this->solicitacao->retornaProdutosSolicitacao($nrorg, $filial["CDFILIAL"], $dataInicial, $dataFinal);    
            }
            
			$response->addDataSet(new DataSet('ProdutosSolicitacao', $produtos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaSolicitacoes(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $solicitacoes = $this->solicitacao->retornaSolicitacoes($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('Solicitacao', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];

            $codigo = $this->solicitacao->retornaProximoCodigoSolicitacao($nrorg, $cdfilial);

			$response->addDataSet(new DataSet('ProximoCodigoSolicitacao', $codigo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function novaSolicitacao(Request\Dataset $request, Response $response) {
        
        try {

            $rows = $request->getDataSet()->getRows();

            $this->solicitacao->novaSolicitacao($rows);

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }

    public function salvaProdutoEditado(Request\Dataset $request, Response $response) {
       
        try {

            $rows = $request->getDataSet()->getRows();

            $produtosJaAprovados = $this->solicitacao->salvaProdutoEditado($rows);

            $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function excluirProdutos(Request\Dataset $request, Response $response) {
       
        try {

            $rows = $request->getDataSet()->getRows();
            $produtosJaAprovados = array();

            foreach ($rows as $row) {

                $produtoJaAprovado = $this->solicitacao->excluirProdutos($row);
                if (!empty($produtoJaAprovado)) {
                    $produtosJaAprovados[] = $produtoJaAprovado;
                }

            }

            $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }
}