<?php
/**
 * Created by Luiz Filho
 */

namespace SCM\Model\Repository;

use Doctrine\ORM\EntityRepository;
use SCM\Traits\TQuery;

class Inventarios extends EntityRepository {
    
    use TQuery;

    public function retornaUltimoInventario($nrorg, $filiais) {
        $dataSet = array();
	  	$sql = "SELECT I.CDFILIAL, I.NRINVENTRETI, NRITINVENTRETI, I.CDPRODUTO, P.NMPRODUTO, I.NRORG
            FROM ITEMINVENTRETI I, PRODUTO P,
            (SELECT MAX(NRINVENTRETI) NRINVENTRETI, CDFILIAL
                    FROM INVENTARIORETI
                    WHERE CDFILIAL = :CDFILIAL
                    AND NRORG      = :NRORG
                    GROUP BY CDFILIAL
                ) NR
            WHERE I.NRINVENTRETI = NR.NRINVENTRETI
            AND I.CDFILIAL  = NR.CDFILIAL
            AND P.CDPRODUTO = I.CDPRODUTO
            AND I.NRORG = :NRORG
            ORDER BY P.NMPRODUTO ASC";

	  	$params = array(
            'NRORG' => $nrorg,
		    'CDFILIAL' => '',
		);
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaProximoCodigoInventario($nrorg, $filiais) {
        $dataSet = array();
        $sql = "SELECT LPAD((NVL(MAX(NRINVENTRETI),0) + 1),10,'0') PROXIMOCODIGO,
            CDFILIAL
            FROM INVENTARIORETI
            WHERE CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
            GROUP BY CDFILIAL";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILIAL' => '',
        );
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaProximoCodigoProdutoInventario($nrorg, $filial, $nrinvent) {
        $dataSet = array();
        $sql = "SELECT LPAD((NVL(MAX(NRITINVENTRETI),0) + 1),4,'0') PROXIMOCODIGO,
            CDFILIAL, NRINVENTRETI
            FROM ITEMINVENTRETI
            WHERE CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
            AND NRINVENTRETI = :NRINVENTRETI
            GROUP BY CDFILIAL, NRINVENTRETI";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILIAL' => $filial,
            'NRINVENTRETI' => $nrinvent,
        );
        // foreach ($filiais as $filial) {
        //     $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function retornaDatasInventarios($nrorg, $filiais) {
        $dataSet = array();
        $sql = "SELECT NRINVENTRETI,
              DSINVENTRETI,
              CDFILIAL,
              TO_CHAR(DTINVENTRETI, 'DD/MM/YYYY') AS DTINVENTRETI
            FROM INVENTARIORETI
            WHERE CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
            ORDER BY TO_DATE(DTINVENTRETI, 'DD/MM/YYYY')";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILIAL' => '',
        );
        
        if (!is_array($filiais)) $filiais = [$filiais];
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        // return $dataSet;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaProdutosInventarios($nrorg, $cdfilial, $nrinvent) {

        $sql = "SELECT ROWNUM, CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, I.CDPRODUTO,
            P.NMPRODUTO, P.CDARVPROD, I.CDOPERINCLUSAO, I.DTINCLUSAO, I.CDOPERULTATU, I.DTULTATU
            FROM ITEMINVENTRETI I, PRODUTO P
            WHERE P.CDPRODUTO = I.CDPRODUTO
            AND I.CDFILIAL  IN ($cdfilial)
            AND I.NRORG = :NRORG
            AND I.NRINVENTRETI = :NRINVENTRETI
            ORDER BY P.NMPRODUTO ASC";

        $params = array(
            'NRORG' => $nrorg,
            'NRINVENTRETI' => $nrinvent
        );

        // $query = $this->getEntityManager()->getConnection()->prepare($sql);
        // $query->execute($params);
        // $result = $query->fetchAll();
        // return $result;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

}