import {
  Loader, IEventParam, Metadata, dayjs as moment, Http, I18n, IDictionary, DatasourceFactory, Config,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, Increment, Button, LoadingService, Form, DialogService, Text,
} from '@zeedhi/common';
import { ScmMask } from '@/config';

export class InventariosController {
  private utilities = Loader.getInstance('Utilities');

  public showInventarioDetail = false;

  public showEditProdutosInventario = false;

  public showAddInventario = false;

  public showAddProduto = false;

  public inventarioDescription = '';

  public inventarioSelected = '';

  public notPossibleDates: string[] = [];

  public openAddInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    form.clearForm();
    this.showAddInventario = true;
    this.setNotPossibleDates();
  }

  get isVisibleInventariosList() {
    return !this.showInventarioDetail && !this.showEditProdutosInventario && !this.showAddInventario && !this.showAddProduto;
  }

  public get setFilterInventarios() {
    return [
      {
        name: 'CDFILIAL',
        value: localStorage.getItem('CDFILIALLOGADA') || '',
        operator: '=',
      },
    ];
  }

  public get setFilterProdutosInventario() {
    return [
      {
        name: 'CDFILIAL',
        value: localStorage.getItem('CDFILIALLOGADA') || '',
        operator: '=',
      },
      {
        name: 'NRINVENTRETI',
        value: this.inventarioSelected || '',
        operator: '=',
      },
    ];
  }

  public clickBackDetailInventario() {
    this.showInventarioDetail = false;
    this.inventarioSelected = '';
  }

  public clickBackEditProdutos() {
    this.showEditProdutosInventario = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickBackAddInventario() {
    this.showAddInventario = false;
  }

  public clickBackAddProduto() {
    this.showAddProduto = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickCardInventario({ component }: IEventParam<Card>) {
    this.showInventarioDetail = true;
    const split = component.name.split('_');
    const numero = split[1];
    const nome = split[2];
    const data = split[3];
    this.inventarioSelected = numero;
    this.inventarioDescription = `${data} - ${nome}`;
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtosRender.reload();
  }

  public async clickCardProdutoInventario({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showEditProdutosInventario = true;
    this.showInventarioDetail = false;
    const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    if (tabsProdutos.tabs.length > 0) {
      // destrói os elementos das tabs que vão ser substituídas para não acontecer conflito de elementos
      await Promise.all(tabsProdutos.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsProdutos.tabs = [];
        tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
      });
    } else tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
    const index = component.name.split('-')[1];
    tabsProdutos.activeTab = await tabsProdutos.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  private newTabs(row: IDictionary) {
    return new Tab({
      name: `${row.CDPRODUTO}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
          component: 'ZdTag',
          tag: 'div',
          children: [
            {
              name: `formEditProduct-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
              component: 'ZdForm',
              children: [
                {
                  allowDuplicate: true,
                  name: 'CDARVPROD',
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Código',
                  value: `${row.CDARVPROD}`,
                  mask: '{{Utilities.getMaskProduto}}',
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'CDPRODUTO',
                  isVisible: false,
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Código',
                  value: `${row.CDPRODUTO}`,
                  mask: '{{Utilities.getMaskProduto}}',
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `nome-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'NMPRODUTO',
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Nome',
                  value: `${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `quantidade-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'QTITINVENTRETI',
                  component: 'ZdIncrement',
                  label: 'Quantidade',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{InventariosController.changeQuantProduct}}',
                  },
                  validations: {
                    required: {},
                    min: {
                      limit: 0.1,
                    },
                  },
                  value: row.QTITINVENTRETI,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'NRITINVENTRETI',
                  component: 'ZdTextInput',
                  isVisible: false,
                  value: `${row.NRITINVENTRETI}`,
                },
              ],
            },
            {
              name: `salvarEditProdutoBtn-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
              component: 'ZdButton',
              label: 'Salvar',
              block: true,
              fixed: true,
              bottom: true,
              disabled: true,
              events: {
                click: '{{InventariosController.salvarEditProduto}}',
              },
              cssStyle: {
                right: '0px',
              },
            },
          ],
        },
      ],
    });
  }

  public setNotPossibleDates() {
    const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
    const cloneConfig = inventarios.datasource.clone();
    const cloneDs = DatasourceFactory.factory(cloneConfig);
    cloneDs.loadAll = true;
    cloneDs.events = {};
    cloneDs.get().then((result: IDictionary) => {
      const inventariosDates = result.map((row: IDictionary) => {
        const dateRow = moment(row.DTINVENTRETI, 'DD/MM/YYYY');
        return dateRow.format('DD/MM/YYYY');
      });
      this.notPossibleDates = inventariosDates;
    });
  }

  public allowedDatesAddInventario(date: string) {
    let valid = true;
    const now = moment();
    const dateVerify = moment(date, 'DD/MM/YYYY');
    if (dateVerify > now) {
      const inventarios = Loader.getInstance('InventariosController');
      if (inventarios.notPossibleDates.includes(dateVerify.format('DD/MM/YYYY'))) valid = false;
    } else valid = false;
    return valid;
  }

  public salvarInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    if (form.validate()) {
      const { value } = form;
      value.CDFILIAL = localStorage.getItem('CDFILIALLOGADA') || '';
      Http.post('/Inventarios/save', { requestType: 'DataSet', dataset: [value] }).then(() => {
        this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
        const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
        inventarios.reload();
        this.clickBackAddInventario();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      });
    }
  }

  public changeQuantProduct({ component }: IEventParam<Increment>) {
    const size = component.parent?.parent?.children?.length || 0;
    const btn = component.parent?.parent?.children[size - 1] as Button;
    btn.disabled = false;
  }

  public adicionarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    form.clearForm();
    this.showInventarioDetail = false;
    this.showAddProduto = true;
  }

  public get setFilterProdutos() {
    return [
      {
        name: 'CDFILIAL',
        value: [localStorage.getItem('CDFILIALLOGADA')] || [],
        operator: '=',
      },
      {
        name: 'CDGRUPPROD',
        value: 'T',
        operator: '=',
      },
    ];
  }

  public clickDeleteInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteInventario',
      type: 'warning',
      title: I18n.translate('ATTENTION'),
      text: I18n.translate('CONFIRM_DELETE_INVENTORY'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('YES'),
          events: { click: () => this.deleteInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteInventario({ component }: IEventParam<Button>) {
    LoadingService.show(' ');
    const nrInvent = component.name.split('-')[1];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: nrInvent,
      },
    ];
    Http.post('/Inventarios/delete', { requestType: 'DataSet', dataset }).then(() => {
      // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
      const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
      inventarios.reload();
      DialogService.hide();
      LoadingService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `${error}`, 'error');
    });
  }

  public clickDeleteProdutoInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteProdutoInventario',
      type: 'warning',
      title: I18n.translate('ATTENTION'),
      text: I18n.translate('CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('YES'),
          events: { click: () => this.deleteProdutoInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteProdutoInventario({ component }: IDictionary) {
    LoadingService.show(' ');
    const nrItInvent = component.name.split('-')[1].split('_')[1];
    const cdroduto = component.name.split('-')[1].split('_')[2];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected,
        NRITINVENTRETI: nrItInvent,
        CDPRODUTO: cdroduto,
      },
    ];
    Http.post('/ProdutosInventario/delete', { requestType: 'DataSet', dataset }).then(() => {
      // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
      const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
      produtos.reload();
      DialogService.hide();
      LoadingService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `${error}`, 'error');
    });
  }

  public salvarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected,
        new: true,
        ...form.value,
      },
    ];
    Http.post('/ProdutosInventario/save', { requestType: 'DataSet', dataset }).then((response: IDictionary) => {
      if (response.data?.length === 0) {
        this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
        form.clearForm();
      } else {
        const mensagem = response.data.error;
        this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
      }
    }).catch((error) => {
      this.utilities.showAlert('alert', error, 'error');
    });
  }

  public salvarEditProduto({ component }: IEventParam<Button>) {
    // const form = Metadata.getInstance<Form>('formEditProduct');
    const form = component.parent?.getChildInstance<Form>(component.parent?.children[0].name);
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected,
        new: false,
        ...form?.value,
      },
    ];
    Http.post('/ProdutosInventario/save', { requestType: 'DataSet', dataset }).then((response: IDictionary) => {
      if (response.data?.length === 0) {
        this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
        component.disabled = true;
        const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
        tabsProdutos.nextTab();
      } else {
        const mensagem = response.data.error;
        this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
      }
    }).catch((error) => {
      this.utilities.showAlert('alert', error, 'error');
    });
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[1].split('_')[2];
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${maskNumber}`;
  }
}
